# -*- coding: utf-8 -*-
from django.contrib import admin
from stored_settings.forms import SettingsCreationForm, SettingsForm
from stored_settings.models import Settings


class SettingsAdmin(admin.ModelAdmin):
    add_form_template = 'admin/stored_settings/settings/create_form.html'
    add_form = SettingsCreationForm

    readonly_fields = ('name', 'key')

    form = SettingsForm
    ordering = ['key']
    list_display = ('name', 'key')

    fieldsets = [
        (None, {
            'fields': ('key', 'name', 'value',),
        })
    ]
    add_view_fieldsets = (
        (None, {
            'classes': ('wide',),
            'fields': ('type', 'key', 'name',)
        }),
    )

    def get_readonly_fields(self, request, obj=None):
        if obj:
            return self.readonly_fields
        return super(SettingsAdmin, self).get_readonly_fields(request, obj)

    def get_fieldsets(self, request, obj=None):
        if not obj:
            return self.add_view_fieldsets
        return super(SettingsAdmin, self).get_fieldsets(request, obj)

    def get_form(self, request, obj=None, **kwargs):
        defaults = {}
        if obj is None:
            defaults.update({
                'form': self.add_form,
                'fields': admin.util.flatten_fieldsets(self.add_view_fieldsets),
            })

        defaults.update(kwargs)
        return super(SettingsAdmin, self).get_form(request, obj, **defaults)

    def response_add(self, request, obj, post_url_continue=None):
        # 'Save' -> 'Save and continue editing'
        if '_addanother' not in request.POST and '_popup' not in request.POST:
            request.POST['_continue'] = 1
        return super(SettingsAdmin, self).response_add(request, obj, post_url_continue)

admin.site.register(Settings, SettingsAdmin)



# -*- coding: utf-8 -*-
from django import forms
from django.contrib.admin.widgets import AdminTextareaWidget, AdminTextInputWidget
from django.forms import widgets
from django.db import models
from django.db.models.fields.files import ImageFieldFile
from django.conf import settings

from stored_settings.models import Settings
from stored_settings.settings import ENABLE_TINYMCE, UPLOAD_TO_DIRECTORY

if ENABLE_TINYMCE:
    assert 'tinymce' in settings.INSTALLED_APPS
    from tinymce.widgets import TinyMCE as HtmlFieldWidget
else:
    HtmlFieldWidget = AdminTextareaWidget


class SettingsCreationForm(forms.ModelForm):
    class Meta:
        model = Settings


class SettingsForm(forms.ModelForm):
    def __init__(self, *args, **kwargs):
        super(SettingsForm, self).__init__(*args, **kwargs)

        if self.instance.type == Settings.Type.CHAR:
            self.fields['value'].widget = AdminTextInputWidget(attrs={})
        elif self.instance.type == Settings.Type.TEXT:
            self.fields['value'].widget = AdminTextareaWidget(attrs={'cols': 90, 'rows': 30},)
        elif self.instance.type == Settings.Type.HTML:
                self.fields['value'].widget = HtmlFieldWidget(attrs={'cols': 90, 'rows': 30},)
        elif self.instance.type == 'image':
            new_field = ImageFieldFile(instance=self.instance, field=models.ImageField(upload_to=UPLOAD_TO_DIRECTORY),
                                       name=self.instance.value)
            self.fields['value'] = forms.ImageField(label=self.fields['value'].label, required=True)
            self.initial['value'] = new_field
        else:
            self.fields['value'].widget = widgets.CheckboxInput(attrs={})

    def save(self, commit=True):
        instance = super(SettingsForm, self).save(commit=False)

        if self.instance.type == Settings.Type.BOOLEAN:
            # if it is checkbox and value is False - save as empty string
            if self.instance.value == 'False':
                self.instance.value = ''

        elif self.instance.type == Settings.Type.IMAGE and 'value' in self.changed_data:
            new_field = ImageFieldFile(instance=self.instance, field=models.ImageField(upload_to=UPLOAD_TO_DIRECTORY),
                                       name=self.instance.value.name)
            new_field.field.name = 'value'
            new_field.save(self.instance.value.name, self.instance.value, True)
            self.instance.value = self.instance.value.lower()

        if commit:
            instance.save()

        return instance



# -*- coding: utf-8 -*-
from django.db.models.fields.files import ImageFieldFile
from django.db. models import ImageField
from stored_settings.models import Settings


def dynamic_settings(request):
    context = {}
    for setting in Settings.objects.all():
        if setting.type == 'box':
            setting.value = bool(setting.value)
        elif setting.type == 'image':
            setting.value = ImageFieldFile(instance=setting, field=ImageField(upload_to='settings'), name=setting.value)

        context[setting.key] = setting.value

    return {'dynamic_settings': context}


"""Module allowing dynamically to create and further on to edit via django-admin settings"""

# About
__author__ = "Vadym Zabiiaka"
__email__ = "v.zabiiaka@gmail.com"
__program__ = "django-stored-settings"
__version__ = "0.1.0"


import os
from os.path import abspath, dirname, join, isfile
from setuptools import setup, find_packages, Command
from setuptools.command.register import register

import stored_settings as module

# Consts
README_MD = "README.md"
README_RST = "README.rst"

# Important vars
cur_dir = abspath(dirname(__file__))
readme_md = join(cur_dir, README_MD)
readme_rst = join(cur_dir, README_RST)

long_description = ""
using_rst = False
try:
    with open(readme_rst, 'r') as f:
        long_description = f.read()
        using_rst = True
except IOError:
    with open(readme_md, 'r') as f:
        long_description = f.read()


class GenerateRstCommand(Command):
    """Generate a README.rst file
    This file can be used after in the register command.
    """

    description = "generate a README.rst file from README.md"
    user_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        if isfile(readme_rst):
            os.remove(readme_rst)

    def run(self):
        import tempfile
        import shutil
        import pypandoc

        tmp_readme_md = join(tempfile.gettempdir(), README_MD)
        shutil.copy(readme_md, tmp_readme_md)
        try:
            # Now, convert to RST
            rst = pypandoc.convert(tmp_readme_md, "rst")
            with open(readme_rst, "w") as f:
                f.write(rst)

        finally:
            os.remove(tmp_readme_md)


class RegisterCommand(register):
    """Check if we're using README.rst before register in Pypi
    """

    def finalize_options(self):
        if not using_rst:
            raise Exception("{} file not found".format(README_RST))

        return register.finalize_options(self)


setup(
    name=module.__program__,
    version=module.__version__,
    description=module.__doc__,
    long_description=long_description,
    author=module.__author__,
    author_email=module.__email__,
    url='https://github.com/zvadim/django-stored-settings',
    download_url='https://github.com/zvadim/django-stored-settings/tarball/%s' % module.__version__,
    keywords=['django', 'settings', 'dynamic', 'stored'],
    packages=find_packages(include=('stored_settings',)),
    package_data={
        'stored_settings': [
            'templates/admin/stored_settings/settings/*.html',
        ],
    },

    requires=['django (>=1.3)'],

    license='MIT license',
    classifiers=[
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Framework :: Django',
        'Topic :: Software Development :: Libraries :: Python Modules'
    ],

    # Commands
    cmdclass={
        "generate_rst": GenerateRstCommand,
        "register": RegisterCommand,
    }
)


# -*- coding: utf-8 -*-
from django.db import models
from django.utils.translation import ugettext_lazy as _


class Settings(models.Model):
    class Type(object):
        CHAR = 'char'
        TEXT = 'text'
        HTML = 'text_html'
        BOOLEAN = 'boolean'
        IMAGE = 'image'

        DEFAULT = CHAR

        CHOICES = (
            (CHAR, _('One line input')),
            (BOOLEAN, _('Checkbox')),
            (IMAGE, _('Image Field')),
            (TEXT, _('Multi-line field')),
            (HTML, _('Multi-line field with HTML'))
        )
    name = models.CharField(_('Help text'), max_length=100)
    key = models.CharField(_('Field key'), max_length=100, unique=True)
    value = models.TextField(_('Value'), blank=True)
    type = models.CharField(_('Field type'), choices=Type.CHOICES, max_length=16, default=Type.DEFAULT)

    def __unicode__(self):
        return '%s (%s)' % (self.name, self.key)

    class Meta:
        verbose_name = _('Setting')
        verbose_name_plural = _('Stored Settings')